﻿//////////////////////////////////////////////////////////////////////////////////////////
//                                                                                      //
//  Boardmodul: Bereitstellung gerätespezifischer Funktionen und Mechanismen            //
//  ________________________________________________________________________            //
//                                                                                      //
//  Copyright (c) 2019 Reimar Grasbon													//
//                                                                                      //
//////////////////////////////////////////////////////////////////////////////////////////
//                                                                                      //
//  This software is subjected to the FreeBSD License. Redistribution and use in source //
//  and binary forms, with or without modification, are permitted provided that the     //
//  following conditions are met:                                                       //
//                                                                                      //
//  1. Redistributions of source code must retain the above copyright notice, this list //
//     of conditions and the following disclaimer.                                      //
//                                                                                      //
//  2. Redistributions in binary form must reproduce the above copyright notice, this   //
//     list of conditions and the following disclaimer in the documentation and/or      //
//     other materials provided with the distribution.                                  //
//                                                                                      //
//  THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS" AND ANY //
//  EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED           //
//  WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED.  //
//  IN NO EVENT SHALL THE COPYRIGHT HOLDER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT,    //
//  INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT  //
//  NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR  //
//  PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY,   //
//  WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE)  //
//  ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE          //
//  POSSIBILITY OF SUCH DAMAGE.                                                         //
//                                                                                      //
//////////////////////////////////////////////////////////////////////////////////////////

#pragma once

#include "TypeDefs.h"
#include "DateTime.h"
#include "AVRChip.h"

//////////////////////////////////////////////////////////////////////////////////////////
//  Typen  ///////////////////////////////////////////////////////////////////////////////
//////////////////////////////////////////////////////////////////////////////////////////
typedef enum        // Allgemeiner Kontakt:
{   GPIO_A0 = 0,    // · An Port-Pin A.0
    GPIO_A1 = 1,    // · An Port-Pin A.1
    GPIO_A2 = 2,    // · An Port-Pin A.2
    GPIO_A3 = 3,    // · An Port-Pin A.3
    GPIO_B0 = 4,    // · An Port-Pin B.0
    GPIO_B4 = 5,    // · An Port-Pin B.4
    GPIO_D4 = 6     // · An Port-Pin D.4
} Gpio;

typedef enum        // Konfiguration eines Port-Pins:
{   PIN_T  = _T_,   // · Eingang, Tristate
    PIN_P  = _P_,   // · Eingang, Pullup
    PIN_L  = _L_,   // · Ausgang, Low
    PIN_H  = _H_,   // · Ausgang, High
} PinConf;

typedef enum                                                      // Note:
{   NOTE_XX  =  0,                                                //  Stille
	NOTE_C3  =  1, NOTE_C3f =  2, NOTE_D3  =  3, NOTE_D3f =  4,   //  C3   C3#  D3   D3#
	NOTE_E3  =  5, NOTE_F3  =  6, NOTE_F3f =  7, NOTE_G3  =  8,   //  E3   F3   F3#  G3
	NOTE_G3f =  9, NOTE_A3  = 10, NOTE_B3  = 11, NOTE_H3  = 12,   //  G3#  A3   B3   H3
	NOTE_C4  = 13, NOTE_C4f = 14, NOTE_D4  = 15, NOTE_D4f = 16,   //  C4   C4#  D4   D4#
    NOTE_E4  = 17, NOTE_F4  = 18, NOTE_F4f = 19, NOTE_G4  = 20,   //  E4   F4   F4#  G4
    NOTE_G4f = 21, NOTE_A4  = 22, NOTE_B4  = 23, NOTE_H4  = 24,   //  G4#  A4   B4   H4
    NOTE_C5  = 25, NOTE_C5f = 26, NOTE_D5  = 27, NOTE_D5f = 28,   //  C5   C5#  D5   D5#
    NOTE_E5  = 29, NOTE_F5  = 30, NOTE_F5f = 31, NOTE_G5  = 32,   //  E5   F5   F5#  G5
    NOTE_G5f = 33, NOTE_A5  = 34, NOTE_B5  = 35, NOTE_H5  = 36,   //  G5#  A5   B5   H5
    NOTE_C6  = 37, NOTE_C6f = 38, NOTE_D6  = 39, NOTE_D6f = 40,   //  C6   C6#  D6   D6#
    NOTE_E6  = 41, NOTE_F6  = 42, NOTE_F6f = 43, NOTE_G6  = 44,   //  E6   F6   F6#  G6
    NOTE_G6f = 45, NOTE_A6  = 46, NOTE_B6  = 47, NOTE_H6  = 48,   //  G6#  A6   B6   H6
    NOTE_C7  = 49, NOTE_C7f = 50, NOTE_D7  = 51, NOTE_D7f = 52,   //  C7   C7#  D7   D7#
    NOTE_E7  = 53, NOTE_F7  = 54, NOTE_F7f = 55, NOTE_G7  = 56,   //  E7   F7   F7#  G7
    NOTE_G7f = 57, NOTE_A7  = 58, NOTE_B7  = 59, NOTE_H7  = 60,   //  G7#  A7   B7   H7
    NOTE_C8  = 61, NOTE_C8f = 62, NOTE_D8  = 63, NOTE_D8f = 64,   //  C8   C8#  D8   D8#
    NOTE_E8  = 65, NOTE_F8  = 66, NOTE_F8f = 67, NOTE_G8  = 68,   //  E8   F8   F8#  G8
    NOTE_G8f = 69, NOTE_A8  = 70, NOTE_B8  = 71, NOTE_H8  = 72,   //  G8#  A8   B8   H8
    NOTE_C9  = 73 									              //  C9
}  Note;

typedef enum
{   KEY_1 = 0,
    KEY_2 = 1
} Key;

typedef enum
{   LED_2,
    LED_3
} Led;

typedef enum                // Signale der System-LED:
{   SYS_LED_OFF       = 0,  // · Ausgeschaltet
    SYS_LED_ON        = 1,  // · Eingeschaltet
    SYS_LED_HEARTBEAT = 2   // · Herzschlag
} SysLedSignal;

//////////////////////////////////////////////////////////////////////////////////////////
//  Funktionen  //////////////////////////////////////////////////////////////////////////
//////////////////////////////////////////////////////////////////////////////////////////
	void          BOARD_Reset(void);
	void          BOARD_EnterPgmLdr(void);
	UInt16        BOARD_MSecTicks(void);
    UInt16        BOARD_SecTicks(void);
	void          BOARD_EEPLoad(const void*, void*, UInt16);
	void		  BOARD_EEPSave(const void*, void*, UInt16);
    void	      BOARD_BusOnActivity(void);
    bool          BOARD_BusIsIdle(void);
    void          BOARD_BusTransmit(UInt08);
    bool          BOARD_BusIsTransmitting(void);
    bool          BOARD_BusTakeFailure(void);
    bool          BOARD_BusReceive(UInt08*);
    bool          BOARD_BusIsReceiving(void);
    void	      BOARD_SerialOnActivity(void);
	void          BOARD_SerialTransmit(UInt08);
	bool          BOARD_SerialReceive(UInt08*);
	void          BOARD_SerialFlush(void);
    void          BOARD_Clock(DateTime*);
    void          BOARD_SetClock(const DateTime*);
    bool          BOARD_IsMuted(void);
    void          BOARD_SetMuted(bool);
    void          BOARD_PlayTone(Note, UInt08);
    bool          BOARD_IsSilence(void);
    UInt08        BOARD_Led1Pwm(void);
    void          BOARD_SetLed1Pwm(UInt08);
    PinConf       BOARD_GpioConfiguration(Gpio);
    void          BOARD_ConfigureGpio(Gpio, PinConf);
    bool          BOARD_Gpio(Gpio);
    void          BOARD_SetGpio(Gpio, bool);
    UInt16        BOARD_AnalogInput(void);
    bool          BOARD_ReadI2C(SInt08, UInt08*, UInt08);
    bool          BOARD_WriteI2C(SInt08, const UInt08*, UInt08);
    bool          BOARD_Led(Led);
    void          BOARD_SetLed(Led, bool);
    bool          BOARD_TakeKey(Key);
    bool          BOARD_Key(Key);
    SysLedSignal  BOARD_SysLedSignal(void);
    void          BOARD_SetSysLedSignal(SysLedSignal);

//////////////////////////////////////////////////////////////////////////////////////////
//  Prozessaufrufe  //////////////////////////////////////////////////////////////////////
//////////////////////////////////////////////////////////////////////////////////////////
	void        BOARD_Startup(void);
	bool        BOARD_DoMainLoop(void);
	void        BOARD_Proceed(void);
	void        BOARD_Shutdown(void);

//////////////////////////////////////////////////////////////////////////////////////////
