﻿//////////////////////////////////////////////////////////////////////////////////////////
//                                                                                      //
//  Serielle Schnittstelle: Ein/Ausgabe von Textzeilen                                  //
//  _____________________________________________________                               //
//                                                                                      //
//  Copyright (c) 2013 Reimar Grasbon													//
//                                                                                      //
//////////////////////////////////////////////////////////////////////////////////////////
//                                                                                      //
//  This software is subjected to the FreeBSD License. Redistribution and use in source //
//  and binary forms, with or without modification, are permitted provided that the     //
//  following conditions are met:                                                       //
//                                                                                      //
//  1. Redistributions of source code must retain the above copyright notice, this list //
//     of conditions and the following disclaimer.                                      //
//                                                                                      //
//  2. Redistributions in binary form must reproduce the above copyright notice, this   //
//     list of conditions and the following disclaimer in the documentation and/or      //
//     other materials provided with the distribution.                                  //
//                                                                                      //
//  THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS" AND ANY //
//  EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED           //
//  WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED.  //
//  IN NO EVENT SHALL THE COPYRIGHT HOLDER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT,    //
//  INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT  //
//  NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR  //
//  PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY,   //
//  WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE)  //
//  ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE          //
//  POSSIBILITY OF SUCH DAMAGE.                                                         //
//                                                                                      //
//////////////////////////////////////////////////////////////////////////////////////////

#include <avr/pgmspace.h>
#include "Conf.h"
#include "SerialLine.h"
#include "Board.h"

//////////////////////////////////////////////////////////////////////////////////////////
//  Daten  ///////////////////////////////////////////////////////////////////////////////
//////////////////////////////////////////////////////////////////////////////////////////
static char    rxLine[CFG_SER_LINE_CAPACITY+1] = "";
static bool    lineComplete                    = false;

//////////////////////////////////////////////////////////////////////////////////////////
//  Lokale Funktionen  ///////////////////////////////////////////////////////////////////
//////////////////////////////////////////////////////////////////////////////////////////
static void
TransmitChar(char c)
{
    BOARD_SerialTransmit((UInt08)c);
}

//////////////////////////////////////////////////////////////////////////////////////////
static bool
ReceiveChar(char *c)
{
    return BOARD_SerialReceive((UInt08*)c);
}

//////////////////////////////////////////////////////////////////////////////////////////
//  Public Functions  ////////////////////////////////////////////////////////////////////
//////////////////////////////////////////////////////////////////////////////////////////
const char*
SERLINE_Receive(void)
{
    if (!lineComplete) return NULL;
    lineComplete = false;
    return rxLine;
}

//////////////////////////////////////////////////////////////////////////////////////////
void
SERLINE_Transmit(const char* line)
{
    BOARD_SerialOnActivity();
    for (UInt08 i=0; line[i] != '\0'; i++)
    {   TransmitChar(line[i]);
    }
    for (UInt08 i=0; CFG_SER_LINE_BREAK[i] != '\0'; i++)
    {   TransmitChar(CFG_SER_LINE_BREAK[i]);
    }
}


//////////////////////////////////////////////////////////////////////////////////////////
//  Prozessaufrufe  //////////////////////////////////////////////////////////////////////
//////////////////////////////////////////////////////////////////////////////////////////
void
SERLINE_Startup(void)
{
}

//////////////////////////////////////////////////////////////////////////////////////////
void
SERLINE_Proceed(void)
{
    static UInt08  col = 0;
    char           c;

    if (lineComplete) return;
    while (ReceiveChar(&c))
    {   if (col == 0) BOARD_SerialOnActivity();
        // (1) Tabulatoren und Vorschübe in Leerzeichen umwandeln
        if (c == '\t' || c == '\v' || c == '\f') c = ' ';
        // (2) Leerzeichen zu Beginn einer Zeile überlesen
        if (col == 0 && c == ' ') continue;
        // (3) Bei zwei aufeinander folgenden Leerzeichen, das zweite ignorieren
        if (c == ' ' && rxLine[col-1] == ' ') continue;
        // (4) Zeilenvorschub und Wagenrücklauf in '\0' umwandeln
        if (c == '\n' || c == '\r') c = '\0';
        // (5) Leere Zeilen ignorieren
        if (col == 0 && c == '\0') continue;
        // (6) Zeichen im Zeilenpuffer ablegen
        if (col < sizeof(rxLine))
        {   rxLine[col] = c;
            col++;
        }
        else
        {   rxLine[0] = '\0';   // Zeile übergelaufen -> Zeile leer
        }
        // (7) Zeilenende erreicht
        if (c == '\0')
        {   if (rxLine[col-1] == ' ') rxLine[col-1] = '\0'; // Ggf. letztes ' ' entf.
            col = 0;
            lineComplete = true;
        }
    }
}

//////////////////////////////////////////////////////////////////////////////////////////
void
SERLINE_Shutdown(void)
{
}

//////////////////////////////////////////////////////////////////////////////////////////
